# Fitting a joint host-symbiont model in Stan
# Maxwell B. Joseph, November 2015

## Initialization ------------------------------------------------
# load data
stan_d <- readRDS('stan_d.rds')
attach(stan_d)

# load helper functions and packages
library(scales)
library(rstan)
library(reshape2)

## Model fitting ------------------------------------------------
# initialize model
mvp_init <- stan("mod.stan", data=stan_d, chains=1, iter=10, cores=1)

# fit the model (takes ~ 24 hrs on a 4 core machine)
mvp_fit <- stan(fit=mvp_init, data=stan_d, chains=4, iter=1200, cores=4,  
                control=list(max_treedepth=13))

# evaluate convergence
summary(mvp_fit)
traceplot(mvp_fit, 'alpha', inc_warmup=F)
traceplot(mvp_fit, c('gamma_1', 'gamma_0'), inc_warmup=F)
traceplot(mvp_fit, 'cor_net', inc_warmup=T)
traceplot(mvp_fit, 'cor_site', inc_warmup=F)
traceplot(mvp_fit, 'cor_indiv', inc_warmup=F)
traceplot(mvp_fit, 'cor_host', inc_warmup=F)
traceplot(mvp_fit, 'lp__', inc_warmup=F)

# evaluate sampler parameters for each chain
lapply(get_sampler_params(mvp_fit, inc_warmup = F), summary, digits = 2)

# create a list of mcmc output
post <- rstan::extract(mvp_fit)

## Figure generation ------------------------------------------------------
## Figure 1: site level correlations -------------------------------------
par(mar=c(1.5, 2, 1.5, 1))
nspec <- H+S
snames <- c('Anbo', 'Psre', 'Lica', 'Tagr', 'Tato', 'Rib', 'Echino', 'Cephalo', 
            'Alaria', 'Rv', 'Bd')
lo <- 20 # length out for sequences of histogram breakpoints
br <- seq(0, max(post$tau_site), length.out=lo)
cor_br <- seq(-1, 1, length.out=lo)
bins <- 40 # number of bins for smoothed scatter plots
alph <- .3 # alpha transparency for points

par(mfrow=c(H+S, H+S), oma=c(0, 0, 0, 0), xpd=TRUE)
for (i in 1:nspec){
  for (j in 1:nspec){
    par(mar=c(1, .8, .5, 0), mgp=c(3, .1, 0))
    if (i == j){
      h <- hist(post$tau_site[,i],
                main="", breaks=br, freq=FALSE,
                col=ifelse(i <= H, "darkolivegreen3", "cornflowerblue"), border=NA,
                ylab="", cex.axis=.5, yaxt='n', tck=-.1, xlab='')
      text(mean(br), median(c(0, max(h$density))), snames[i], cex=1.4)
    } else {
      if (j > i){
        p_hi <- mean(post$cor_site[, i, j] > 0)
        p_lo <- mean(post$cor_site[, i, j] < 0)
        prob_extreme <- p_hi > .95 | p_lo > .95
        hist(post$cor_site[, i, j], main="", breaks=cor_br,
             ylab="", col=ifelse(prob_extreme, "black", "grey"),
             border=NA, cex.axis=.5, yaxt='n', xaxt='n')
        axis(1, at=c(-1, 0, 1), cex.axis=.5, tck=-.1)
      } else {
        par(mar=c(rep(.8, 4)), bty='n')
        i_meds <- apply(post$alpha_site[, , i], 2, mean)
        j_meds <- apply(post$alpha_site[, , j], 2, mean)
        plot(i_meds, j_meds, col=alpha(1, alph + .1), 
             cex=.4, yaxt="n", xaxt="n")
      }
    }
  }
}

## Figure 2: within-site, among survey correlations --------------------------
par(mfrow=c(H, H))
for (i in 1:H){
  for (j in 1:H){
    par(mar=c(1.8, .5, .5, .1), mgp=c(3, .6, 0))
    if (i == j){
      h <- hist(post$tau_net[,i], col="darkolivegreen3",
                main="", border=NA,
                breaks=seq(0, max(post$tau_net), length.out=lo),
                freq=FALSE, yaxt='n', tck=-.05, cex.axis=.8)
      text(1.5, median(c(0, max(h$density))), snames[i], cex=1.7)
    } else {
      if (j > i){
        p_hi <- mean(post$cor_net[, i, j] > 0)
        p_lo <- mean(post$cor_net[, i, j] < 0)
        prob_extreme <- p_hi > .95 | p_lo > .95
        hist(post$cor_net[, i, j], main="", breaks=cor_br, ylab="", border=NA,
             col=ifelse(prob_extreme, "black", "grey"),
             yaxt='n', xaxt='n')
        axis(1, at=c(-1, 0, 1), cex.axis=.8, tck=-.05)
      } else {
        par(mar=rep(.8, 4), bty='n')
        i_meds <- apply(post$alpha_net[, , i], 2, mean)
        j_meds <- apply(post$alpha_net[, , j], 2, mean)
        plot(i_meds, j_meds, col=alpha(1, alph), cex=.4, yaxt="n", xaxt="n")
      }
    }
  }
}

## Figure 3: among host species correlations ---------------------------------
par(mfrow=c(S, S))
scheme <- c('#66c2a5', '#fc8d62', '#8da0cb', '#e78ac3', '#a6d854')
for (i in 1:S){
  for (j in 1:S){
    par(mar=c(1.8, .8, .5, 0), mgp=c(3, .5, 0))
    if (i == j){
      h <- hist(post$tau_host[,i], col="dodgerblue", main='',
                breaks=seq(0, max(post$tau_host), length.out=lo),
                freq=FALSE, yaxt='n', tck=-.1, cex.axis=.8, border=NA)
      text(4, median(c(0, max(h$density))), snames[H + i], cex=1.7)
    } else {
      if (j > i){
        p_hi <- mean(post$cor_host[, i, j] > 0)
        p_lo <- mean(post$cor_host[, i, j] < 0)
        prob_extreme <- p_hi > .95 | p_lo > .95
        hist(post$cor_host[, i, j], main="", breaks=cor_br, ylab="", 
             col=ifelse(prob_extreme, "black", "grey"),
             freq=FALSE, yaxt='n', tck=-.1, cex.axis=1, border=NA, xaxt='n')
        axis(1, at=c(-1, 0, 1), cex.axis=.8, tck=-.1)
      } else {
        par(mar=rep(.8, 4), bty='n')
        i_meds <- apply(post$alpha_host[, , i], 2, mean)
        j_meds <- apply(post$alpha_host[, , j], 2, mean)
        vals <- cbind(c(post$alpha_host[, , i]), c(post$alpha_host[, , j]))
        smoothScatter(vals, nrpoints=0, xlab="", ylab="", nbin=bins,
                      colramp = colorRampPalette(c("white", "grey")),
                      yaxt="n", xaxt="n")
        text(i_meds, j_meds, snames[1:H], cex=.8, col=alpha(1, .8))
      }
    }
  }
}

## Figure 4: among individual correlations -----------------------------------
par(mfrow=c(S, S))
for (i in 1:S){
  for (j in 1:S){
    par(mar=c(1.5, .8, .5, 0), mgp=c(3, .5, 0))
    if (i == j){
      h <- hist(post$tau_indiv[,i], col="dodgerblue", main='',
                breaks=seq(0, ceiling(max(post$tau_indiv)), length.out=lo),
                freq=FALSE, yaxt='n', tck=-.1, cex.axis=.8, border=NA)
      text(2.5, median(c(0, max(h$density))), snames[H + i], cex=1.7)
    } else {
      if (j > i){
        p_hi <- mean(post$cor_indiv[, i, j] > 0)
        p_lo <- mean(post$cor_indiv[, i, j] < 0)
        prob_extreme <- p_hi > .95 | p_lo > .95
        hist(post$cor_indiv[, i, j], main="", breaks=cor_br, 
             ylab="", col=ifelse(prob_extreme, "black", "grey"),
             freq=FALSE, yaxt='n', tck=-.1, border=NA, xaxt='n')
        axis(1, at=c(-1, 0, 1), cex.axis=.8, tck=-.1)
      } else {
        par(mar=c(rep(.8, 4)), bty='n')
        i_meds <- apply(post$alpha_indiv[, , i], 2, mean)
        j_meds <- apply(post$alpha_indiv[, , j], 2, mean)
        plot(i_meds, j_meds, col=alpha(1, .1), 
             cex=.4, yaxt="n", xaxt="n")
      }
    }
  }
}

# Figure 5: effective variance & dependence ----------------------------------
post$V_e <- matrix(nrow=length(post$lp__), ncol=4)
post$D_e <- matrix(nrow=length(post$lp__), ncol=4)
vars <- c('site', 'host', 'net', 'indiv')
colnames(post$V_e) <- vars
colnames(post$D_e) <- vars
for (i in 1:length(post$V_e[, 1])){
  for (j in 1:length(vars)){
    tau <- paste0('tau_', vars[j])
    corn <- paste0('cor_', vars[j])
    Sigma <- diag(post[[tau]][i, ]) %*% 
      post[[corn]][i, , ] %*% 
      diag(post[[tau]][i, ])
    post$V_e[i, j] <- det(Sigma) ^ (1/length(post[[tau]][i, ]))
    post$D_e[i, j] <- 1 - det(post[[corn]][i, , ]) ^ (1/length(post[[tau]][i, ]))
  }
}

rv <- range(post$V_e)
rd <- range(post$D_e)

par(mfrow=c(1, 1), mar=c(5, 4, 4, 2) + 0.1)
alph <- .2
plot(post$V_e[, 'host'], post$D_e[, 'host'], main='', 
     xlim=rv, ylim=rd, col=alpha(3, alph + .1), pch=1,
     xlab='Effective variance', ylab='Effective dependence')
points(post$V_e[, 'site'], post$D_e[, 'site'], col=alpha(1, alph), pch=1)
points(post$V_e[, 'net'], post$D_e[, 'net'], col=alpha(2, alph), pch=1)
points(post$V_e[, 'indiv'], post$D_e[, 'indiv'], col=alpha(4, alph), pch=1)
legend('topright', col=c(3, 1, 2, 4), pch=1, 
       legend=c('Among host species', 'Among sites', 
                'Among nets', 'Among individuals'), 
       bty='n')
