data {
  int<lower=0> nsite; // number of sites
  int<lower=1> H; // number of host species

  // host abundance data and indices
  int<lower=0> N_h; // number of host abundance observations
  int<lower=0> site_h[N_h]; // site indices for host abundance data
  int<lower=0> spec_h[N_h]; // species indices for host abundance data
  int<lower=0> n_net; // number of nets included
  int<lower=0> net_id[N_h]; // net id
  int<lower=0> y_h[N_h]; // host count data
  int<lower=0, upper=1> seen_h[nsite, H];
  int index_h[nsite, H];
  int length_h[nsite, H];

  // symbiont data and indices
  int<lower=1> S; // number of symbionts
  int<lower=1> N_s; // length, total observation vector
  int<lower=1, upper=nsite> site_s[N_s];
  int<lower=1, upper=S> s_s[N_s];
  int<lower=1, upper=H> h_s[N_s];
  int<lower=1> n_indiv;
  int<lower=1, upper=n_indiv> id_s[N_s];
  int<lower=0> y_s[N_s];
  int<lower=0, upper=1> seen_s[nsite, S];
  int index_s[nsite, S];
  int length_s[nsite, S];
}

parameters {
  // abundance model:
  // intercept parameters
  real<lower=0> sd_alpha;
  real mu_alpha;
  vector[H + S] z_alpha;

  // random effects: abundance
  cholesky_factor_corr[H] L_net;
  matrix[H, n_net] z_net;
  vector[H] tau_netR;

  cholesky_factor_corr[H + S] L_site;
  matrix[H + S, nsite] z_site;
  vector[H + S] tau_siteR;

  cholesky_factor_corr[S] L_indiv;
  matrix[S, n_indiv] z_indiv;
  vector[S] tau_indivR;

  cholesky_factor_corr[S] L_host;
  matrix[S, H] z_host;
  vector[S] tau_hostR;

  // hyperparameters
  real mu_net;
  real mu_site;
  real mu_indiv;
  real mu_host;
  real<lower=0> sigma_tau;
  real mu_gamma0;
  real<lower=0> sigma_gamma0;

  // host occurrence params
  vector[H + S] gamma_0R;
  vector<lower=0>[2] gamma_1;
}

transformed parameters{
  // parameter declaration block
  vector[H + S] alpha;
  matrix[n_net, H] alpha_net;
  matrix[nsite, H + S] alpha_site;
  matrix[n_indiv, S] alpha_indiv;
  matrix[H, S] alpha_host;
  vector<lower=0>[H] tau_net;
  vector<lower=0>[H+S] tau_site;
  vector<lower=0>[S] tau_indiv;
  vector<lower=0>[S] tau_host;
  vector[N_h] loglambda_h;
  vector[N_s] loglambda_s;

  // occupancy parameters
  vector[H + S] gamma_0;
  matrix[nsite, H] psi_h;
  matrix[nsite, S] psi_s;

  // log likelihoods
  matrix[nsite, H] ll_h;
  matrix[nsite, S] ll_s;
  // end parameter declaration

  // impose location and scale on random effects (Matt trick)
  alpha <- mu_alpha + sd_alpha * z_alpha;
  gamma_0 <- mu_gamma0 + sigma_gamma0 * gamma_0R;
  tau_net <- exp(mu_net + sigma_tau * tau_netR);
  tau_site <- exp(mu_site + sigma_tau * tau_siteR);
  tau_indiv <- exp(mu_indiv + sigma_tau * tau_indivR);
  tau_host <- exp(mu_host + sigma_tau * tau_hostR);

  // mvnorm random effects
  alpha_net <- (diag_pre_multiply(tau_net, L_net) * z_net)';
  alpha_site <- (diag_pre_multiply(tau_site, L_site) * z_site)';
  alpha_indiv <- (diag_pre_multiply(tau_indiv, L_indiv) * z_indiv)';
  alpha_host <- (diag_pre_multiply(tau_host, L_host) * z_host)';

  // host expectations
  for (i in 1:N_h){
  // log expected density
    loglambda_h[i] <- alpha[spec_h[i]] +
                        alpha_site[site_h[i], spec_h[i]] +
                        alpha_net[net_id[i], spec_h[i]];
  }

  for (i in 1:N_s){
    loglambda_s[i] <- alpha[H+s_s[i]]
        + alpha_site[site_s[i], H + s_s[i]]
        + alpha_indiv[id_s[i], s_s[i]]
        + alpha_host[h_s[i], s_s[i]];
  }

  // logit-probability of occurrence
  for (i in 1:nsite){
    for (j in 1:H){
      psi_h[i, j] <- inv_logit(gamma_0[j] + gamma_1[1] *
                                  (alpha[j] + alpha_site[i, j]));
    }
    for (j in 1:S){
      psi_s[i, j] <- inv_logit(gamma_0[H+j] + gamma_1[2] *
                                  (alpha[H+j] + alpha_site[i, H+j]));
    }
  }

  for (i in 1:nsite){
    // host likelihood
    for (j in 1:H){
      if (seen_h[i, j] == 1){
        ll_h[i, j] <- poisson_log_log(
            segment(y_h, index_h[i, j], length_h[i, j]),
            segment(loglambda_h, index_h[i, j], length_h[i, j]))
            + log(psi_h[i, j]);
      } else {
        ll_h[i, j] <- log_sum_exp(
          poisson_log_log(
            segment(y_h, index_h[i, j], length_h[i, j]),
            segment(loglambda_h, index_h[i, j], length_h[i, j]))
            + log(psi_h[i, j]), log1m(psi_h[i, j]));
      }
    }

    // symbiont likelihood
    for (j in 1:S){
      if (seen_s[i, j] == 1){
        ll_s[i, j] <- poisson_log_log(
            segment(y_s, index_s[i, j], length_s[i, j]),
            segment(loglambda_s, index_s[i, j], length_s[i, j]))
            + log(psi_s[i, j]);
      } else {
        ll_s[i, j] <- log_sum_exp(
          poisson_log_log(
            segment(y_s, index_s[i, j], length_s[i, j]),
            segment(loglambda_s, index_s[i, j], length_s[i, j]))
            + log(psi_s[i, j]), log1m(psi_s[i, j]));
      }
    }
  }
}

model {
  // intercepts
  mu_alpha ~ normal(0, 1);
  sd_alpha ~ normal(0, 1);
  z_alpha ~ normal(0, 1);

  // occupancy-density parameters
  mu_gamma0 ~ normal(0, 1);
  sigma_gamma0 ~ normal(0, 1);
  gamma_0R ~ normal(0, 1);
  gamma_1 ~ normal(1, 1);

  // net level random effects
  L_net ~ lkj_corr_cholesky(2);
  tau_netR ~ normal(0, 1);
  to_vector(z_net) ~ normal(0, 1);

  // site level random effects
  L_site ~ lkj_corr_cholesky(2);
  tau_siteR ~ normal(0, 1);
  to_vector(z_site) ~ normal(0, 1);

  // individual level random effects
  L_indiv ~ lkj_corr_cholesky(2);
  tau_indivR ~ normal(0, 1);
  to_vector(z_indiv) ~ normal(0, 1);

  // host-species random effects
  L_host ~ lkj_corr_cholesky(2);
  tau_hostR ~ normal(0, 1);
  to_vector(z_host) ~ normal(0, 1);

  // hyperparameters
  mu_net ~ normal(0, .5);
  mu_site ~ normal(0, .5);
  mu_indiv ~ normal(0, .5);
  mu_host ~ normal(0, .5);
  sigma_tau ~ normal(0, .5);

  increment_log_prob(sum(ll_h));
  increment_log_prob(sum(ll_s));
}

generated quantities {
  matrix[H, H] cor_net;
  matrix[H + S, H + S] cor_site;
  matrix[S, S] cor_indiv;
  matrix[S, S] cor_host;
  vector[nsite * H + nsite * S] log_lik;

  cor_net <- multiply_lower_tri_self_transpose(L_net);
  cor_site <- multiply_lower_tri_self_transpose(L_site);
  cor_indiv <- multiply_lower_tri_self_transpose(L_indiv);
  cor_host <- multiply_lower_tri_self_transpose(L_host);
  log_lik <- append_row(to_vector(ll_h), to_vector(ll_s));
}
